package org.avaje.metric.filereport;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Handles the file directory, name and writer etc.
 */
public class FileOutput {

  private static final Logger logger = Logger.getLogger(FileOutput.class.getName());
  
  protected Writer writer;
  
  public FileOutput() {
    
    File f = new File(getDirectory(), getFileName());
    File parentFile = f.getParentFile();
    if (parentFile != null && !parentFile.exists()) {
      parentFile.mkdirs();
    }
    try {
      FileWriter fileWriter = new FileWriter(f, true);
      writer= new BufferedWriter(fileWriter, 1024);
      
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
  
  public Writer getWriter() {
    return writer;
  }
  
  public void close() {
    try {
      writer.flush();
    } catch (IOException e) {
      logger.log(Level.SEVERE, "Failed to flush metric file writer", e);
    }
    try {
      writer.close();
    } catch (IOException e){
      logger.log(Level.SEVERE, "Failed to close metric file writer", e);
    }
  }
  
  private File getDirectory() {
    
    String metricDirectory = System.getProperty("metric.directory");
    if (metricDirectory == null) {
      return new File(".");
    } else {
      return new File(metricDirectory);
    }
  }
  
  private String getFileName() {
    
    String todayString = new SimpleDateFormat("yyyyMMdd").format(new Date());
    return "metric-"+todayString+".txt";
  }
  
}
