package org.avaje.metric.filereport;

import java.io.IOException;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.avaje.metric.Metric;
import org.avaje.metric.MetricManager;

/**
 * Writes the collected metrics to a file.
 */
public class FileReporter {

  private static final Logger logger = Logger.getLogger(FileReporter.class.getName());
  
  protected final SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
  
  protected final Timer timer = new Timer("MetricFileReporter", true);
  
  protected final int freqInSeconds;
  
  protected Writer writer;
  
  public static FileReporter everyOneMinute() {
    return new FileReporter(60);
  }
  
  public static FileReporter everyFiveMinutes() {
    return new FileReporter(5*60);
  }
  
  public FileReporter(int freqInSeconds) {
    this.freqInSeconds = freqInSeconds;
    long freqMillis = freqInSeconds*1000;
    this.timer.scheduleAtFixedRate(new WriteTask(), freqMillis, freqMillis);
  }
  
  private class WriteTask extends TimerTask {
    public void run() {
      try {
        writeAll();
      } catch (IOException e) {
        logger.log(Level.SEVERE, "", e);
      }
    }
  }
  
  public void writeAll() throws IOException {
  
    FileOutput fo = new FileOutput();
    try {
      writer = fo.getWriter();
      
      TextFileWriteVisitor visitor = new TextFileWriteVisitor(writer, freqInSeconds);
      visitor.onStartVisiting();
      
      Collection<Metric> allMetrics = MetricManager.getAllMetrics();
      for (Metric metric : allMetrics) {
        metric.visit(visitor);
      }
      
    } finally {
      fo.close();
    }
  }
  
}
